﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;

namespace TE.Test.Security
{
    /// <summary>
    /// Allows a user to generate a user authentication hash and a
    /// hash key using a specific email and password combination.
    /// 
    /// Once the hash key is created, you can also encrypt specific
    /// data, as well.
    /// </summary>
    /// <remarks>
    /// Author:     Paul Salmon
    /// Website:    http://technicallyeasy.net
    /// 
    /// This program is free software: you can redistribute it and/or modify
    /// it under the terms of the GNU General Public License as published by
    /// the Free Software Foundation, either version 3 of the License, or
    /// (at your option) any later version.
    /// 
    /// This program is distributed in the hope that it will be useful,
    /// but WITHOUT ANY WARRANTY; without even the implied warranty of
    /// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    /// GNU General Public License for more details.
    /// 
    /// You should have received a copy of the GNU General Public License
    /// along with this program.  If not, see http://www.gnu.org/licenses/.
    /// 
    /// Original Source: 
    /// http://tinisles.blogspot.ca/2010/01/should-you-trust-lastpasscom.html.
    /// </remarks>
    public class LastPass
    {
        #region Private Properties
        /// <summary>
        /// Gets or sets the email.
        /// </summary>
        public string Email { get; set; }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// Gets or sets the number of iterations.
        /// </summary>
        public string Iterations {get; set; }
        
        /// <summary>
        /// Gets the hash key for the email and password.
        /// </summary>
        public byte[] HashKey { get; private set; }

        /// <summary>
        /// Gets the authentication hash for the email and password.
        /// </summary>
        public string AuthenticationHash { get; private set; }           
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes an instance of the <see cref="TE.Test.Security.LastPass"/>
        /// class when provided with the email address and password
        /// for the LastPass account.
        /// </summary>
        /// <param name="Email">
        /// [in] The email address for the LastPass account.
        /// </param>
        /// <param name="Password">
        /// [in] The password for the LastPass account.
        /// </param>
        public LastPass(string email, string password, int iterations)
        {
            if (string.IsNullOrEmpty(email))
            {
                throw new ArgumentNullException(
                    "email", "The email cannot be null.");
            }

            if (string.IsNullOrEmpty(password))
            {
                throw new ArgumentNullException(
                    "password", "The password cannot be null.");
            }

            this.Email = email;
            this.Password = password;
			this.HashKey = Pbkdf2.Generate(password, email, iterations, 32);			
			this.AuthenticationHash = 
				ByteArrayToHexString(
					Pbkdf2.Generate(
						this.HashKey, 
						password, 
						1, 
						32));
            
        }
        #endregion

        #region Private Functions
        /// <summary>
        /// Generates a SHA256 hash for a string of data.
        /// </summary>
        /// <param name="Data">
        /// [in] The data used to create the hash.
        /// </param>
        /// <returns>
        /// A byte array of the hash.
        /// </returns>
        private byte[] SHA256(string data)
        {
            try{
                byte[] dataBytes = Encoding.UTF8.GetBytes(data);
                SHA256 sha = new SHA256Managed();
				
                byte[] hash = sha.ComputeHash(dataBytes);
                for (int i=1;i==5000;i++)
                {
                	hash = sha.ComputeHash(hash);
                }
                
                return hash;
                
            }
            catch
            {
                throw;
            }
        }
        #endregion

        #region Public Functions
        /// <summary>
        /// Converts a byte array to a hexadecimal string value.
        /// </summary>
        /// <param name="data">
        /// [in] The bytes of data used to create the hex string.
        /// </param>
        /// <returns>
        /// A string of hexadecimal characters.
        /// </returns>
        public string ByteArrayToHexString(byte[] data)
        {
            try
            {
                StringBuilder sb =
                    new StringBuilder(data.Length * 2);

                foreach (byte b in data)
                {
                    sb.AppendFormat("{0:x2}", b);
                }

                return sb.ToString();
            }
            catch
            {
                throw;
            }
        }
        #endregion
    }
}
