﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;
using TE.Test.Security;

namespace TE.Test
{
    /// <summary>
    /// A form that generates LastPass user data for 
    /// testing/comparing.
    /// </summary>
    /// <remarks>
    /// Author:     Paul Salmon
    /// Website:    http://technicallyeasy.net
    ///
    /// This program is free software: you can redistribute it and/or modify
    /// it under the terms of the GNU General Public License as published by
    /// the Free Software Foundation, either version 3 of the License, or
    /// (at your option) any later version.
    /// 
    /// This program is distributed in the hope that it will be useful,
    /// but WITHOUT ANY WARRANTY; without even the implied warranty of
    /// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    /// GNU General Public License for more details.
    /// 
    /// You should have received a copy of the GNU General Public License
    /// along with this program.  If not, see http://www.gnu.org/licenses/.
    /// </remarks>
    public partial class frmLastPassTest : Form
    {
        #region Private Variables
        /// <summary>
        /// A LastPass object.
        /// </summary>
        private LastPass _account = null;
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes an instance of the <see cref="TE.Test.frmLastPasstest"/>
        /// form.
        /// </summary>
        public frmLastPassTest()
        {
            InitializeComponent();

            // Display or hide the password dending on the value of
            // the checkbox
            this.txtPassword.UseSystemPasswordChar =
                this.chkHide.Checked;

            // Setup the link at the bottom
            lnkTechnicallyEasy.Links.Add(
                 0,
                 lnkTechnicallyEasy.Text.Length,
                 "http://technicallyeasy.net");

        }
        #endregion

        #region Events
        /// <summary>
        /// Closes the form.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            try
            {
                this.Close();
            }
            catch { }
        }

        /// <summary>
        /// Generate the LastPass user data - authentication hash and
        /// the hash key - by using the email and password.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void btnGenerate_Click(object sender, EventArgs e)
        {
            // Flag indicating the user data was generated 
            // successfully
            bool success = false;

            try
            {
                // Generate the information for the user
                this._account = new LastPass(
                    txtEmail.Text,
                    txtPassword.Text,
                    Convert.ToInt32(txtIterations.Text));

                // Display the authentication hash
                txtAuthenticationHash.Text = 
                    this._account.AuthenticationHash;

                // Display the hash key
                txtKey.Text =
                    this._account.ByteArrayToHexString(
                        this._account.HashKey);

                success = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    "Error: " + ex.Message, 
                    "Error", 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }
            finally
            {
                grpLastPassData.Enabled = success;
            }
        }

        /// <summary>
        /// Displays or hides the password.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void chkHide_CheckedChanged(object sender, EventArgs e)
        {
            this.txtPassword.UseSystemPasswordChar =
                this.chkHide.Checked;
        }

        /// <summary>
        /// Opens the link in the default Web browser.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void lnkTechnicallyEasy_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start(
                    e.Link.LinkData.ToString());                
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    "Error: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Selects all text in the textbox when it receives the
        /// focus.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void txtEmail_Enter(object sender, EventArgs e)
        {
            BeginInvoke((Action)delegate
            {
                this.txtEmail.SelectAll();
            });            
        }

        /// <summary>
        /// Enables or disables the generate button.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void txtEmail_TextChanged(object sender, EventArgs e)
        {
            btnGenerate.Enabled =
                (this.txtEmail.Text.Length > 0) &&
                (this.txtPassword.Text.Length > 0);
        }

        /// <summary>
        /// Selects all text in the textbox when it receives the
        /// focus.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void txtPassword_Enter(object sender, EventArgs e)
        {
            BeginInvoke((Action)delegate
            {
                txtPassword.SelectAll();
            });              
        }

        /// <summary>
        /// Enables or disables the generate button.
        /// </summary>
        /// <param name="sender">
        /// [in] Event-related parameter.
        /// </param>
        /// <param name="e">
        /// [in] Event-related parameter.
        /// </param>
        private void txtPassword_TextChanged(object sender, EventArgs e)
        {
            btnGenerate.Enabled =
                (this.txtEmail.Text.Length > 0) &&
                (this.txtPassword.Text.Length > 0);
        }
        #endregion
    }
}
